<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Zk_queController extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Zk_queModel', 'zk_que');
        $this->load->model('UserModel', 'user');
        $this->load->library('Middleware');
        $this->load->helper('url');
        $this->load->library('form_validation');
        $this->output->set_content_type('application/json');
    }

    /**
     * Helper method to send JSON responses.
     */
    protected function jsonResponse($status, $message, $data = [], $httpCode = 200)
    {
        $this->output->set_status_header($status);
        $this->output
            ->set_output(json_encode([
                'status' => $status,
                'message' => $message,
                'data' => $data
            ]));
    }

    /**
     * List all records.
     */
    public function index()
    {
        try {
            $records = $this->zk_que->getAll();
            $this->jsonResponse(200, 'Records retrieved successfully', $records);
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    /**
     * Get pending records.
     */
    public function getPending()
    {
        try {
            $records = $this->zk_que->getPending();
            $this->jsonResponse(200, 'Pending records retrieved successfully', $records);
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    public function returnToPending($id)
    {
        try {
            $this->zk_que->returnToPending($id);
            $this->jsonResponse(200, 'Record returned to pending successfully');
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    /**
     * Get a single record by ID.
     */
    public function show($id)
    {
        try {
            $record = $this->zk_que->getById($id);
            if ($record) {
                $this->jsonResponse(200, 'Record retrieved successfully', $record);
            } else {
                $this->jsonResponse(500, 'Record not found', [], 404);
            }
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    /**
     * Create a new record.
     */
    public function create()
    {

        try {
            // Set validation rules
            $this->form_validation->set_rules('task_type', 'task type', 'required');
            $this->form_validation->set_rules('task_data', 'task data', 'required');
            // Add more validation rules as needed

            if ($this->form_validation->run() == FALSE) {
                $this->jsonResponse(500, validation_errors(), [], 400);
                return;
            }

            // Prepare data
            $data = [
                'task_type' => $this->input->post('task_type'), // XSS filtering
                'task_data' => $this->input->post('task_data')
            ];

            // Create record
            $id = $this->zk_que->create($data);
            $this->jwtmiddleware->insertMutedNotification();

            if ($id) {
                $this->jsonResponse(200, 'Record created successfully', ['id' => $id], 201);
            } else {
                $this->jsonResponse(500, 'Failed to create record', [], 500);
            }
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    public function create_batch()
    {
        try {
            $data = [
                'task_type' => $this->input->post('task_type'),
                'task_data' => $this->input->post('task_data')
            ];

            $task_data = json_decode($data['task_data'], true);
            if (!is_array($task_data)) {
                $this->jsonResponse(500, 'Invalid task_data', [], 400);
                return;
            }
            foreach ($task_data as $item) {
                // Re-encode the item as JSON before saving
                $item_json = json_encode($item, JSON_UNESCAPED_UNICODE);
                $inserted[] = $this->zk_que->create(['task_type' => $data['task_type'], 'task_data' => $item_json]);
            }

            $this->jwtmiddleware->insertMutedNotification();


            if (count($inserted) > 0) {
                $this->jsonResponse(200, 'Record created successfully', ['ids' => $inserted], 201);
            }
            $this->jsonResponse(500, 'Failed to create record', [], 500);
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    /**
     * Update an existing record.
     */
    public function update($id)
    {
        try {
            // Set validation rules
            $this->form_validation->set_rules('status', 'status is required', 'required');
            // Add more validation rules as needed

            if ($this->form_validation->run() == FALSE) {
                $this->jsonResponse(500, validation_errors(), [], 400);
                return;
            }

            // Prepare data
            $data = [
                'status' => $this->input->post('status', TRUE), // XSS filtering
                'message' => $this->input->post('message', TRUE) ?? NULL
                // Add more fields as needed
            ];

            // Update record
            $result = $this->zk_que->update($id, $data);
            if ($result) {
                $this->jwtmiddleware->insertMutedNotification();
                $this->jsonResponse(200, 'Record updated successfully', $data);
            } else {
                $this->jsonResponse(500, 'Failed to update record', [], 500);
            }
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    /**
     * Update the status of a record.
     */
    public function updateStatus($id)
    {
        try {
            // Set validation rules
            $this->form_validation->set_rules('status', 'Status', 'required');

            if ($this->form_validation->run() == FALSE) {
                $this->jsonResponse(500, validation_errors(), [], 400);
                return;
            }

            // Prepare data
            $data = [
                'status' => $this->input->post('status', TRUE), // XSS filtering
            ];

            // Update status
            $result = $this->zk_que->updateStatus($id, $data);
            if ($result) {
                $this->jsonResponse(200, 'Status updated successfully', $data);
            } else {
                $this->jsonResponse(500, 'Failed to update status', [], 500);
            }
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }

    /**
     * Delete a record.
     */
    public function delete($id)
    {
        try {
            $result = $this->zk_que->delete($id);
            if ($result) {
                $this->jsonResponse(200, 'Record deleted successfully');
            } else {
                $this->jsonResponse(500, 'Failed to delete record', [], 500);
            }
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }
}