<?php
defined('BASEPATH') or exit('No direct script access allowed');

class DocumentServiceConventor extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        // Load required models/helpers
        $this->load->model('DocumentConfiguration');
        $this->load->model('DocumentJwtManager');
        $this->load->model('DocumentFileUtils');  // assumed model for file utility functions
    }

    /**
     * Convert a file using the document converter service.
     *
     * @param string $docUri  The URL to the document.
     * @param string $fromExt The source file extension.
     * @param string $toExt   The target file extension.
     * @param string $docKey  The document key.
     * @param bool   $isAsync Whether to run asynchronously.
     * @param string|null $filePass Optional file password.
     * @param string|null $lang     Optional language/region.
     * @return array|null Returns an array with 'uri' and 'fileType' if conversion is finished.
     * @throws Exception If the conversion service returns an error.
     */
    public function getConvertedData($docUri, $fromExt, $toExt, $docKey, $isAsync, $filePass = null, $lang = null)
    {
        // If no file extension was passed, get it from the URI
        if (empty($fromExt)) {
            $fromExt = $this->DocumentFileUtils->getFileExt($docUri);
        }

        // Get the file title (name) using the file utility
        $title = $this->DocumentFileUtils->getFileName($docUri);

        // Build the payload array
        $payload = array(
            'url' => $docUri,
            'outputtype' => str_replace('.', '', $toExt),
            'filetype' => str_replace('.', '', $fromExt),
            'title' => $title,
            'key' => $docKey,
            'password' => $filePass,
            'region' => $lang
        );

        // Add "async" flag if the conversion is asynchronous
        if ($isAsync) {
            $payload['async'] = true;
        }

        // Set up the default headers
        $headers = array(
            'Accept: application/json'
        );

        // If JWT is enabled, encode the tokens and add the header
        if ($this->DocumentJwtManager->isEnabled() && $this->DocumentJwtManager->useForRequest()) {
            // Create a header token that wraps the payload
            $headerToken = $this->DocumentJwtManager->encode(array('payload' => $payload));
            // Also encode the payload as a body token
            $payload['token'] = $this->DocumentJwtManager->encode($payload);
            // Add the Authorization header with the token prefixed by "Bearer "
            $jwtHeader = $this->DocumentConfiguration->jwt_header();
            $headers[] = $jwtHeader . ': Bearer ' . $headerToken;
        }

        // Get the converter URL from the configuration
        $converterUrl = $this->DocumentConfiguration->document_server_converter_url();
        // If the returned value is an array (from parse_url), rebuild it to a string URL
        if (is_array($converterUrl)) {
            $converterUrl = $this->unparse_url($converterUrl);
        }

        // Use cURL to perform a POST request to the converter service
        $ch = curl_init($converterUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        // Send payload as JSON
        $jsonPayload = json_encode($payload);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonPayload);
        // Set SSL verification based on configuration
        $verifyPeer = $this->DocumentConfiguration->ssl_verify_peer_mode_enabled();
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $verifyPeer);
        // Set timeout (in seconds)
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);

        $response = curl_exec($ch);
        $httpStatus = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        // Check if the conversion service returned a 200 OK status
        if ($httpStatus != 200) {
            throw new RuntimeException("Conversion service returned status: " . $httpStatus);
        }

        $jsonResponse = json_decode($response, true);
        return $this->getResponseUri($jsonResponse);
    }

    /**
     * Process the JSON response from the converter service and return the converted file details.
     *
     * @param array $json The decoded JSON response.
     * @return array|null Array with keys 'uri' and 'fileType' if conversion has ended.
     * @throws Exception If the response contains an error.
     */
    public function getResponseUri($json)
    {
        $isEnd = isset($json['endConvert']) ? $json['endConvert'] : false;
        $error = isset($json['error']) ? $json['error'] : null;

        if ($error) {
            $this->processError($error);
        }

        if ($isEnd) {
            return array(
                'uri' => isset($json['fileUrl']) ? $json['fileUrl'] : null,
                'fileType' => isset($json['fileType']) ? $json['fileType'] : null
            );
        }

        // You might choose to return null or an empty array if conversion hasn't finished
        return null;
    }

    /**
     * Process the error code returned by the converter service.
     *
     * @param mixed $error The error code.
     * @throws Exception With an appropriate error message.
     */
    public function processError($error)
    {
        $prefix = 'Error occurred in the ConvertService: ';

        $mapping = array(
            '-9' => $prefix . 'Error conversion output format',
            '-8' => $prefix . 'Error document VKey',
            '-7' => $prefix . 'Error document request',
            '-6' => $prefix . 'Error database',
            '-5' => $prefix . 'Incorrect password',
            '-4' => $prefix . 'Error download error',
            '-3' => $prefix . 'Error conversion error',
            '-2' => $prefix . 'Error conversion timeout',
            '-1' => $prefix . 'Error conversion unknown'
        );

        if (isset($mapping[(string) $error])) {
            throw new Exception($mapping[(string) $error]);
        } else {
            throw new Exception("Error Code: " . $error);
        }
    }

    /**
     * Helper function to reconstruct a URL from its parsed components.
     *
     * @param array $parsed_url The array returned by parse_url().
     * @return string The reconstructed URL.
     */
    private function unparse_url($parsed_url)
    {
        $scheme = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : '';
        $host = isset($parsed_url['host']) ? $parsed_url['host'] : '';
        $port = isset($parsed_url['port']) ? ':' . $parsed_url['port'] : '';
        $user = isset($parsed_url['user']) ? $parsed_url['user'] : '';
        $pass = isset($parsed_url['pass']) ? ':' . $parsed_url['pass'] : '';
        $pass = ($user || $pass) ? "$pass@" : '';
        $path = isset($parsed_url['path']) ? $parsed_url['path'] : '';
        $query = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : '';
        $fragment = isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : '';
        return "$scheme$user$pass$host$port$path$query$fragment";
    }
}
