<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Format_model extends CI_Model
{

    // Cache the loaded formats
    private $_formats = null;

    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Returns all formats loaded from the JSON file.
     *
     * @return array
     */
    public function get_all()
    {
        if ($this->_formats === null) {
            $this->_formats = $this->_load_formats();
        }
        return $this->_formats;
    }

    /**
     * Loads the JSON file and decodes its contents.
     *
     * @return array Array of format objects.
     */
    private function _load_formats()
    {
        $path = $this->_get_file_path();
        if (!file_exists($path)) {
            show_error("Formats file not found at: " . $path, 500);
        }
        $contents = file_get_contents($path);
        $data = json_decode($contents);
        if (json_last_error() !== JSON_ERROR_NONE) {
            show_error("Error decoding JSON: " . json_last_error_msg(), 500);
        }
        return $data;
    }

    /**
     * Returns the full file path for the JSON file.
     *
     * Adjust this path as necessary. Here we assume the assets folder is in the project root.
     *
     * @return string
     */
    private function _get_file_path()
    {
        return FCPATH . 'assets/document-formats/onlyoffice-docs-formats.json';
    }

    /**
     * Helper function that returns the file extension for a given format.
     *
     * @param object $format
     * @return string
     */
    private function _extension($format)
    {
        return '.' . $format->name;
    }

    /*---------------------------
      Filtering Functions
    ---------------------------*/

    /**
     * Returns formats that have the "fill" action.
     *
     * @return array
     */
    public function fillable()
    {
        return array_filter($this->get_all(), function ($format) {
            return in_array('fill', $format->actions);
        });
    }

    /**
     * Returns the file extensions for fillable formats.
     *
     * @return array
     */
    public function fillable_extensions()
    {
        return array_map([$this, '_extension'], $this->fillable());
    }

    /**
     * Returns formats that have the "view" action.
     *
     * @return array
     */
    public function viewable()
    {
        return array_filter($this->get_all(), function ($format) {
            return in_array('view', $format->actions);
        });
    }

    /**
     * Returns the file extensions for viewable formats.
     *
     * @return array
     */
    public function viewable_extensions()
    {
        return array_map([$this, '_extension'], $this->viewable());
    }

    /**
     * Returns formats that can be edited (either "edit" or "lossy-edit" actions).
     *
     * @return array
     */
    public function editable()
    {
        return array_filter($this->get_all(), function ($format) {
            return in_array('edit', $format->actions) || in_array('lossy-edit', $format->actions);
        });
    }

    /**
     * Returns the file extensions for editable formats.
     *
     * @return array
     */
    public function editable_extensions()
    {
        return array_map([$this, '_extension'], $this->editable());
    }

    /**
     * Returns formats that support auto-conversion.
     *
     * @return array
     */
    public function convertible()
    {
        return array_filter($this->get_all(), function ($format) {
            return in_array('auto-convert', $format->actions);
        });
    }

    /**
     * Returns the file extensions for convertible formats.
     *
     * @return array
     */
    public function convertible_extensions()
    {
        return array_map([$this, '_extension'], $this->convertible());
    }

    /**
     * Returns formats where type is 'pdf'.
     *
     * @return array
     */
    public function pdfs()
    {
        return array_filter($this->get_all(), function ($format) {
            return isset($format->type) && $format->type === 'pdf';
        });
    }

    /**
     * Returns the file extensions for PDF formats.
     *
     * @return array
     */
    public function pdf_extensions()
    {
        return array_map([$this, '_extension'], $this->pdfs());
    }

    /**
     * Returns formats where type is 'cell' (spreadsheet).
     *
     * @return array
     */
    public function spreadsheets()
    {
        return array_filter($this->get_all(), function ($format) {
            return isset($format->type) && $format->type === 'cell';
        });
    }

    /**
     * Returns the file extensions for spreadsheet formats.
     *
     * @return array
     */
    public function spreadsheet_extensions()
    {
        return array_map([$this, '_extension'], $this->spreadsheets());
    }

    /**
     * Returns formats where type is 'slide' (presentation).
     *
     * @return array
     */
    public function presentations()
    {
        return array_filter($this->get_all(), function ($format) {
            return isset($format->type) && $format->type === 'slide';
        });
    }

    /**
     * Returns the file extensions for presentation formats.
     *
     * @return array
     */
    public function presentation_extensions()
    {
        return array_map([$this, '_extension'], $this->presentations());
    }

    /**
     * Returns formats where type is 'word' (document).
     *
     * @return array
     */
    public function documents()
    {
        return array_filter($this->get_all(), function ($format) {
            return isset($format->type) && $format->type === 'word';
        });
    }

    /**
     * Returns the file extensions for document formats.
     *
     * @return array
     */
    public function document_extensions()
    {
        return array_map([$this, '_extension'], $this->documents());
    }

    /**
     * Returns file extensions for all formats.
     *
     * @return array
     */
    public function all_extensions()
    {
        return array_map([$this, '_extension'], $this->get_all());
    }
}
