const emailsShouldNotifiy = require("./constants");
const sendMail = require("./email");
const db = require("./db");

require("dotenv").config();


// check fingerprints every finger print in exits on the database

const checkFingerPrintsHealth = async () => {
    try {
        const sql1 = `
            SELECT * FROM emp_fingerprint ef
            LEFT JOIN finger_print_scheduler fps ON fps.fingerprint_in_id = ef.id
            WHERE fps.user_hash IS NULL
            AND ef.in_out = 'in'
            ORDER BY ef.fingerprint_time DESC
        `;

        const sql2 = `
            SELECT * FROM emp_fingerprint ef
            LEFT JOIN finger_print_scheduler fps ON fps.fingerprint_out_id = ef.id
            WHERE fps.user_hash IS NULL
            AND ef.in_out = 'out'
            ORDER BY ef.fingerprint_time DESC
        `;

        // Execute both queries
        const [rows1] = await db.promise().query(sql1);
        const [rows2] = await db.promise().query(sql2);

        // Combine and log results
        let hasIssues = false;

        if (rows1.length > 0) {
            hasIssues = true;
            console.log("Incorrect 'in' records found:");
            const incorrectInData = rows1.map(row => ({
                fingerprintId: row.id,
                fingerprintTime: row.fingerprint_time,
                inOut: row.in_out,
                userHash: row.user_hash
            }));
            console.log(incorrectInData);
        }

        if (rows2.length > 0) {
            hasIssues = true;
            console.log("Incorrect 'out' records found:");
            const incorrectOutData = rows2.map(row => ({
                fingerprintId: row.id,
                fingerprintTime: row.fingerprint_time,
                inOut: row.in_out,
                userHash: row.user_hash
            }));
            console.log(incorrectOutData);
        }

        // Notify if issues exist
        if (hasIssues) {
            console.log("Sending email notifications...");
            for (const email of emailsShouldNotifiy) {
                try {
                    const emailContent = `
                        Incorrect 'in' records: ${JSON.stringify(rows1, null, 2)}
                        Incorrect 'out' records: ${JSON.stringify(rows2, null, 2)}
                    `;
                    await sendMail(
                        email,
                        "Database Alert: Incorrect Data Found fingerprints",
                        emailContent
                    );
                    console.log(`Email sent to ${email}`);
                } catch (mailErr) {
                    console.error(`Error sending email to ${email}:`, mailErr);
                }
            }
        } else {
            console.log("No incorrect data found.");
        }
    } catch (err) {
        console.error("Error monitoring the database:", err);
    }
};


const checkEmployeesWorkedHoursHealth = async () => {
    try {
        const ewhIssuesQuery = `
            SELECT * FROM spaceLight.employees_worked_hours 
            LEFT JOIN daily_flexible_schedule_targets 
                ON employees_worked_hours.date = daily_flexible_schedule_targets.date 
                AND daily_flexible_schedule_targets.user_hash = employees_worked_hours.user_hash
            LEFT JOIN actual_schedules 
                ON employees_worked_hours.date = actual_schedules.date 
                AND actual_schedules.user_hash = employees_worked_hours.user_hash
            WHERE is_flexible = '0'  
                AND daily_flexible_schedule_targets.id IS NOT NULL
                AND actual_schedules.id IS NULL
            ORDER BY employees_worked_hours.id DESC
        `;

        const [ewhRows] = await db.promise().query(ewhIssuesQuery);

        if (ewhRows.length > 0) {
            console.log("Employees Worked Hours Issues Found:");
            const ewhIssueData = ewhRows.map(row => ({
                date: row.date,
                userHash: row.user_hash,
                isFlexible: row.is_flexible
            }));
            console.log(ewhIssueData);

            // Send email notifications for EWH issues
            for (const email of emailsShouldNotifiy) {
                try {
                    const emailContent = `
                        Employees Worked Hours Issues: 
                        ${JSON.stringify(ewhIssueData, null, 2)}
                    `;
                    await sendMail(
                        email,
                        "Database Alert: Employees Worked Hours Problems",
                        emailContent
                    );
                    console.log(`Email sent to ${email} for EWH issues`);
                } catch (mailErr) {
                    console.error(`Error sending email to ${email} for EWH issues:`, mailErr);
                }
            }
        } else {
            console.log("No Employees Worked Hours issues found.");
        }
    } catch (err) {
        console.error("Error checking Employees Worked Hours:", err);
    }
};





const bootstrap = async () => {
    try {
        await checkFingerPrintsHealth();
        await checkEmployeesWorkedHoursHealth();

        console.log("DATA BASE CONNECTION CLOSED");
    } catch (err) {
        console.error("Error closing the database connection:", err);
    } finally {
        await db.end();
    }
}


bootstrap();
