<?php
$config_path = '/var/www/html/spaceLight/app/application/config/database.php';
// $config_path = '../app/application/config/database.php';

// Check if the CodeIgniter database configuration file exists
if (file_exists($config_path)) {
    // Include the configuration file
    include $config_path;

    // Debugging: Check if $db is actually an array
    if (!is_array($db)) {
        die("Error: \$db is not an array. The config file might not be loaded correctly.");
    }

    // Ensure active group is set
    $active_group = isset($active_group) ? $active_group : 'default';

    // Check if the active group exists in $db
    if (!isset($db[$active_group])) {
        die("Error: Active group '$active_group' does not exist in \$db.");
    }

    // Continue with database connection using $db[$active_group]
    $host = $db[$active_group]['hostname'];
    $db_name = $db[$active_group]['database'];
    $user = $db[$active_group]['username'];
    $pass = $db[$active_group]['password'];
    $charset = 'utf8mb4'; // Define your charset

    // Create DSN and set PDO options
    $dsn = "mysql:host=$host;dbname=$db_name;charset=$charset";
    $options = [
        PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
        PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
        PDO::ATTR_EMULATE_PREPARES => false,
    ];

    // Try establishing a PDO connection
    try {
        $pdo = new PDO($dsn, $user, $pass, $options);
        echo "Database connection successful!";
    } catch (PDOException $e) {
        // Log connection error
        error_log("Database connection failed: " . $e->getMessage(), 3, './error_log.txt');
        die('Database connection failed.');
    }
} else {
    error_log("Error: Configuration file not found" . $e->getMessage(), 3, './error_log.txt');

    die('Error: Configuration file not found.');
}

function fetchFingerprints($pdo, $userHash, $previousDate, $date)
{
    try {
        // Query to get all fingerprint entries for the previous date and the specified date
        $sql = "
            SELECT
                fingerprint_time,
                in_out
            FROM
                emp_fingerprint
            WHERE
                user_hash = :user_hash
                AND (
                    DATE(fingerprint_time) = :previous_date
                    OR DATE(fingerprint_time) = :date
                )
                AND is_flexible = '1'
            ORDER BY fingerprint_time ASC
        ";

        $stmt = $pdo->prepare($sql);
        $stmt->execute([
            'user_hash' => $userHash,
            'previous_date' => $previousDate,
            'date' => $date,
        ]);

        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        echo "Error: " . $e->getMessage();
        return [];
    }
}

// Function to calculate total worked hours based on in and out times
function calculateWorkedHours($inTimes, $outTimes)
{
    $totalWorkedSeconds = 0;

    // Pair up in and out times
    for ($i = 0; $i < min(count($inTimes), count($outTimes)); $i++) {
        $inTime = new DateTime($inTimes[$i]);
        $outTime = new DateTime($outTimes[$i]);
        $interval = $inTime->diff($outTime);
        $totalWorkedSeconds += $interval->h * 3600 + $interval->i * 60 + $interval->s;
    }

    return $totalWorkedSeconds / 3600; // Convert seconds to hours
}

// Function to fetch the target hours for a specific user and date
function fetchTargetHours($pdo, $userHash, $date)
{
    $query = "
        SELECT target_hours
        FROM daily_flexible_schedule_targets
        WHERE user_hash = :user_hash
        AND date = :date
        AND is_deleted = '0'
    ";

    $stmt = $pdo->prepare($query);
    $stmt->execute([
        'user_hash' => $userHash,
        'date' => $date,
    ]);

    return $stmt->fetchColumn();
}

// Function to insert a calculated clock-out time if there are remaining hours
function insertClockOutTime($pdo, $userHash, $clockOutTime)
{
    $query = "
        INSERT INTO emp_fingerprint (user_hash, fingerprint_time, in_out, is_flexible)
        VALUES (:user_hash, :clock_out_time, 'out', '1')
    ";

    $stmt = $pdo->prepare($query);
    $stmt->execute([
        'user_hash' => $userHash,
        'clock_out_time' => $clockOutTime,
    ]);
}

// Main function to process fingerprint data and handle missing clock-out entries
function processFingerprints($pdo, $userHash, $date)
{
    // Fetch fingerprint data for both the specified date and the day before
    $previousDate = date('Y-m-d', strtotime($date . ' -1 day'));
    $fingerprints = fetchFingerprints($pdo, $userHash, $previousDate, $date);

    $inTimes = [];
    $outTimes = [];

    // Categorize fingerprint times
    foreach ($fingerprints as $fingerprint) {
        if ($fingerprint['in_out'] === 'in') {
            $inTimes[] = $fingerprint['fingerprint_time'];
        } elseif ($fingerprint['in_out'] === 'out') {
            $outTimes[] = $fingerprint['fingerprint_time'];
        }
    }

    // Ensure times are in chronological order
    sort($inTimes);
    sort($outTimes);

    // Calculate worked hours
    $workedHours = calculateWorkedHours($inTimes, $outTimes);
    $targetHours = fetchTargetHours($pdo, $userHash, $date);

    $remainingHours = max($targetHours - $workedHours, 0);

    // Output the results for debugging or logging purposes
    echo "In Times: " . print_r($inTimes, true);
    echo "Out Times: " . print_r($outTimes, true);
    echo "Total Worked Hours: " . number_format($workedHours, 2) . " hours\n";
    echo "Target Hours: " . $targetHours . " hours\n";
    echo "Remaining Hours: " . number_format($remainingHours, 2) . " hours\n";

    // Insert clock-out record
    if ($remainingHours > 0) {
        $latestClockIn = new DateTime(end($inTimes));
        $clockOutTime = $latestClockIn->add(new DateInterval('PT' . ($remainingHours * 3600) . 'S'))->format('Y-m-d H:i:s');

        insertClockOutTime($pdo, $userHash, $clockOutTime);

        echo "Inserted Clock-Out Time: " . $clockOutTime . "\n";
    } else {
        // Clock out 5 minutes after last clock-in if no remaining hours
        $latestClockIn = new DateTime(end($inTimes));
        $clockOutTime = $latestClockIn->add(new DateInterval('PT5M'))->format('Y-m-d H:i:s');

        insertClockOutTime($pdo, $userHash, $clockOutTime);

        echo "Inserted Clock-Out Time (5 minutes after clock-in): " . $clockOutTime . "\n";
    }
}

// Function to retrieve users who need a clock-out entry for the previous day
function fetchUsersWithMissingClockOut($pdo, $yesterday)
{
    $query = "
        SELECT
            f1.user_hash,
            MAX(f1.fingerprint_time) AS latest_clock_in,
            COUNT(CASE WHEN f1.in_out = 'in' AND f1.is_flexible = '1' THEN 1 END) AS clock_in_count,
            COUNT(CASE WHEN f1.in_out = 'out' THEN 1 END) AS clock_out_count,
            t.target_hours,
            :yesterday1 AS date
        FROM
            emp_fingerprint f1
        JOIN
            daily_flexible_schedule_targets t
            ON f1.user_hash = t.user_hash
            AND DATE(f1.fingerprint_time) = t.date
            AND t.is_deleted = '0'
        WHERE
            (
                (DATE(f1.fingerprint_time) = :yesterday2)
                OR (DATE(f1.fingerprint_time) = CURDATE() AND TIME(f1.fingerprint_time) <= '04:00:00')
            )
            AND f1.is_flexible = '1'
        GROUP BY
            f1.user_hash, t.target_hours
        HAVING
            clock_in_count > clock_out_count
    ";

    $stmt = $pdo->prepare($query);
    $stmt->execute([
        'yesterday1' => $yesterday,
        'yesterday2' => $yesterday,
    ]);

    return $stmt->fetchAll(PDO::FETCH_ASSOC);
}

function fetchUsersWithMissingClockOutFIX($pdo, $startDate, $endDate)
{
    $query = "
        SELECT
            f1.user_hash,
            MAX(f1.fingerprint_time) AS latest_clock_in,
            COUNT(CASE WHEN f1.in_out = 'in' AND f1.is_flexible = '1' THEN 1 END) AS clock_in_count,
            COUNT(CASE WHEN f1.in_out = 'out' THEN 1 END) AS clock_out_count,
            t.target_hours,
            DATE(f1.fingerprint_time) AS date  -- Added date selection
        FROM
            emp_fingerprint f1
        JOIN
            daily_flexible_schedule_targets t
            ON f1.user_hash = t.user_hash
            AND DATE(f1.fingerprint_time) = t.date
            AND t.is_deleted = '0'
        WHERE
            (DATE(f1.fingerprint_time) BETWEEN :startDate AND :endDate)
            OR (DATE(f1.fingerprint_time) = :endDate AND TIME(f1.fingerprint_time) <= '04:00:00')
            AND f1.is_flexible = '1'
        GROUP BY
            f1.user_hash, t.target_hours
        HAVING
            clock_in_count > clock_out_count
    ";

    $stmt = $pdo->prepare($query);

    // Parameters to bind
    $params = [
        'startDate' => $startDate,
        'endDate' => $endDate,
    ];

    // Bind parameters with debugging output
    foreach ($params as $key => $value) {
        echo "Binding parameter $key with value: $value\n";
        $stmt->bindValue(":$key", $value);
    }

    // Execute the statement and handle exceptions
    try {
        $stmt->execute();
    } catch (Exception $e) {
        echo "Execution Error: " . $e->getMessage();
        return []; // Return empty array on error
    }

    return $stmt->fetchAll(PDO::FETCH_ASSOC);
}

function fetchUsersWithMissingClockOutFIXSAD($pdo, $startDate, $endDate)
{
    $query = "
        SELECT
            f1.user_hash,
            MAX(f1.fingerprint_time) AS latest_clock_in,
            COUNT(CASE WHEN f1.in_out = 'in' AND f1.is_flexible = '1' THEN 1 END) AS clock_in_count,
            COUNT(CASE WHEN f1.in_out = 'out' THEN 1 END) AS clock_out_count,
            t.target_hours,
            :currentDate1 AS date
        FROM
            emp_fingerprint f1
        JOIN
            daily_flexible_schedule_targets t
            ON f1.user_hash = t.user_hash
            AND DATE(f1.fingerprint_time) = t.date
            AND t.is_deleted = '0'
        WHERE
            (
                (DATE(f1.fingerprint_time) = :currentDate2)
                OR (DATE(f1.fingerprint_time) = CURDATE() AND TIME(f1.fingerprint_time) <= '04:00:00')
            )
            AND f1.is_flexible = '1'
        GROUP BY
            f1.user_hash, t.target_hours
        HAVING
            clock_in_count > clock_out_count
    ";

    $stmt = $pdo->prepare($query);

    $date = $startDate;
    $result = [];

    while (strtotime($date) <= strtotime($endDate)) {
        $stmt->execute([
            'currentDate1' => $date,
            'currentDate2' => $date,
        ]);

        $result = array_merge($result, $stmt->fetchAll(PDO::FETCH_ASSOC));
        $date = date('Y-m-d', strtotime($date . ' +1 day'));
    }

    return $result;
}

try {
    $startDate = '2024-09-01';
    $endDate = '2024-09-28';

    $usersWithMissingClockOut = fetchUsersWithMissingClockOutFIXSAD($pdo, $startDate, $endDate);

    foreach ($usersWithMissingClockOut as $user) {
        processFingerprints($pdo, $user['user_hash'], $user['date']);
    }

} catch (PDOException $e) {
    echo "Error: " . $e->getMessage();
}

// Main script execution
// try {
//     $yesterday = date('Y-m-d', strtotime('-1 day'));

//     $usersWithMissingClockOut = fetchUsersWithMissingClockOut($pdo, $yesterday);

//     foreach ($usersWithMissingClockOut as $user) {
//         processFingerprints($pdo, $user['user_hash'], $user['date']);
//     }

// } catch (PDOException $e) {
//     echo "Error: " . $e->getMessage();
// }

// try {
//     $startDate = '2024-09-15';
//     $endDate = '2024-09-24';

//     // Fetch users with missing clock-out times
//     $usersWithMissingClockOut = fetchUsersWithMissingClockOutFIX($pdo, $startDate, $endDate);

//     // Process each user
//     foreach ($usersWithMissingClockOut as $user) {
//         // Make sure 'date' key exists before accessing
//         if (isset($user['date'])) {
//             processFingerprints($pdo, $user['user_hash'], $user['date']);
//         } else {
//             echo "User {$user['user_hash']} does not have a date associated.\n";
//         }
//     }
// } catch (Exception $e) {
//     // Output the error message
//     echo 'Error: ' . $e->getMessage();
// }
